#ifndef _plumb__defines_h_
#define _plumb__defines_h_

typedef short plumb_int_t;
#define plumb_chr yyctx->chr
#define plumb_val yyctx->val
#define plumb_lval yyctx->lval
#define plumb_stack yyctx->stack
#define plumb_debug yyctx->debug
#define plumb_nerrs yyctx->nerrs
#define plumb_errflag yyctx->errflag
#define plumb_state yyctx->state
#define plumb_yyn yyctx->yyn
#define plumb_yym yyctx->yym
#define plumb_jump yyctx->jump
#line 24 "plumb_gram.y"

#define IN_LEX
#define IN_GRAM
#include <stdlib.h>
#include "plumb_gram.y.tab.h"
#define YYSTYPE plumb_STYPE

#include "fd.h"
#include "gram_execute.h"
#include "parse_common.h"
#include "plumb.h"
#include "var.h"

#line 39 "plumb_gram.y"
typedef union plumb_tokunion_u {
	char *str;
	int num;

	exe_fd_t *fd;       /* fd reference (proc/fd) */
	char *proc;         /* name of the process */
	kv_t *kv;           /* key = value */
	kvlist_t *kvlist;   /* list of key-value pairs */
} plumb_tokunion_t;
typedef plumb_tokunion_t plumb_STYPE;


#define NL 257
#define KW_START 258
#define KW_STOP 259
#define KW_KILL 260
#define KW_PAUSE 261
#define KW_RESUME 262
#define KW_FLUSH 263
#define KW_EXIT 264
#define KW_DUMP_INTERNALS 265
#define KW_QUERY 266
#define INT 267
#define STRING 268
#define plumb_ERRCODE 256

#ifndef plumb_INITSTACKSIZE
#define plumb_INITSTACKSIZE 200
#endif

typedef struct {
	unsigned stacksize;
	plumb_int_t *s_base;
	plumb_int_t *s_mark;
	plumb_int_t *s_last;
	plumb_STYPE *l_base;
	plumb_STYPE *l_mark;
#if plumb_DEBUG
	int debug;
#endif
} plumb_STACKDATA;

typedef struct {
	int errflag;
	int chr;
	plumb_STYPE val;
	plumb_STYPE lval;
	int nerrs;
	int yym, yyn, state;
	int jump;
	int stack_max_depth;
	int debug;

	/* variables for the parser stack */
	plumb_STACKDATA stack;
} plumb_yyctx_t;

typedef enum { plumb_RES_NEXT, plumb_RES_DONE, plumb_RES_ABORT } plumb_res_t;

extern int plumb_parse_init(plumb_yyctx_t *yyctx);
extern plumb_res_t plumb_parse(plumb_yyctx_t *yyctx, plumb_ctx_t *ctx, int tok, plumb_STYPE *lval);
extern void plumb_error(plumb_ctx_t *ctx, plumb_STYPE tok, const char *msg);


#endif /* _plumb__defines_h_ */
