/*  plumb - the pipe shell - affix: prefix or suffix records
    Copyright (C) 2012-2013  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page: http://repo.hu/projects/plumb
    Contact: plumb (at) igor2.repo.hu
*/

#include <stdlib.h>
#include <string.h>
#include "process.h"
#include "fd.h"
#include "libporty/util/buf.h"
#include "virt.h"

typedef struct {
	int len;
	char append;
	char *buff;
	int alloced;
	char affix[1];
} affix_t;

static virt_ctrl_t affix_input(proc_virt_t *proc, int local_fd, const char *data, int len)
{
	int tlen;
	affix_t *a = proc->data;

	tlen = len + a->len;
	if (tlen > a->alloced) {
		a->alloced = tlen + 256;
		a->buff = realloc(a->buff, a->alloced);
	}
	if (a->append) {
		memcpy(a->buff, data, len);
		memcpy(a->buff+len, a->affix, a->len);
	}
	else {
		memcpy(a->buff, a->affix, a->len);
		memcpy(a->buff+a->len, data, len);
	}
	vp_write(proc, 1, a->buff, tlen);

	return VC_OK;
}

void affix_free(proc_virt_t *p)
{
	affix_t *a = p->data;
	if (a->buff != NULL)
		free(a->buff);
	free(a);
}

int vp_affix_init(proc_virt_t *p, kvlist_t *args)
{
	affix_t *a;
	int len;
	char append;
	char *suffix = NULL, *prefix = NULL, *affix;
	vp_args_t vargs[] = {
		{"suffix", &suffix},
		{"prefix", &prefix},
		{NULL, NULL}
	};

	vp_stdfds_set(p, VP_REQ_QUIT_ON_UNBIND, VP_REQ_QUIT_ON_UNBIND, VP_REQ_UNBOUND, VP_REQ_UNBOUND, VP_REQ_UNBOUND, VP_REQ_UNBOUND);

	vp_args(args, vargs);

	if ((prefix == NULL) && (suffix == NULL))
		plumb_error_("affix %s without prefix or suffix\n", p->h.id);

	if ((prefix != NULL) && (suffix != NULL))
		plumb_error_("affix %s with both prefix and suffix\n", p->h.id);

	if (prefix != NULL) {
		append = 0;
		affix = prefix;
	}
	else {
		append = 1;
		affix = suffix;
	}

	len = strlen(affix);
	a = malloc(sizeof(affix_t) + len);
	a->buff = NULL;
	a->alloced = 0;
	a->len = len;
	a->append = append;
	memcpy(a->affix, affix, len);

	p->input = affix_input;
	p->free  = affix_free;
	p->data  = a;

	return 0;
}
